**** QR and IVQR Regression
* We also thank David Autor and Sari Pekkala Kerr for
* sharing their Matlab code for the Chernozhukov-Hansen IVQR estimator.
* Data Set: data_for_ivqr_0.dta

* 1: Boys
use data_for_ivqr_0, clear
keep if sex==1
drop sex
save data_for_ivqr_1, replace
* 2: Girls
use data_for_ivqr_0, clear
keep if sex==0
drop sex
save data_for_ivqr_2, replace
* 3: Front Rows
use data_for_ivqr_0, clear
keep if srow<=5
save data_for_ivqr_3, replace
* 4: Rear Rows
use data_for_ivqr_0, clear
keep if srow>5
save data_for_ivqr_4, replace
* 5: English
use data_for_ivqr_0, clear
keep if english==1
save data_for_ivqr_5, replace
* 6: Korean
use data_for_ivqr_0, clear
keep if english==0
save data_for_ivqr_6, replace
* 7: Econ
use data_for_ivqr_0, clear
keep if major==1
drop bus humsoc nateng exch
save data_for_ivqr_7, replace
* 8: Non-Econ
use data_for_ivqr_0, clear
keep if major~=1
drop bus
save data_for_ivqr_8, replace

clear
!erase qr_result.txt

use data_for_ivqr_0, clear /* change the name of file according to subgroup */
outfile final using y.txt, nolabel replace wide
outfile navgmid1_pair - srow using x.txt, nolabel replace wide
outfile mid pairmid using d.txt, nolabel replace
outfile tgpa ptgpa misstgpa missptgpa using z.txt, nolabel replace

* Run matlab code
winexec matlab -r chivqreg2d_ivqr
*shell matlab -r chivqreg2d

capture confirm file qr_result.txt

while _rc {      /* file does not exist! */
capture confirm file qr_result.txt
}

/***** Results *****/
insheet qr_mid_est qr_pmid_est qr_mid_se qr_pmid_se ///
		ivqr_mid_est ivqr_pmid_est ivqr_mid_se ivqr_pmid_se temp using qr_result.txt, tab clear

drop temp

* Calculate 95% Confidence Interval
gen qr_mid_c0 = qr_mid_est - 1.644853626951 * qr_mid_se
gen qr_mid_c1 = qr_mid_est + 1.644853626951 * qr_mid_se

gen qr_pmid_c0 = qr_pmid_est - 1.644853626951 * qr_pmid_se
gen qr_pmid_c1 = qr_pmid_est + 1.644853626951 * qr_pmid_se

gen ivqr_mid_c0 = ivqr_mid_est - 1.644853626951 * ivqr_mid_se
gen ivqr_mid_c1 = ivqr_mid_est + 1.644853626951 * ivqr_mid_se

gen ivqr_pmid_c0 = ivqr_pmid_est - 1.644853626951 * ivqr_pmid_se
gen ivqr_pmid_c1 = ivqr_pmid_est + 1.644853626951 * ivqr_pmid_se

aorder

save qr_result_0, replace  /* change the name of file according to subgroup */

gen qt = _n
gen y = 0

* Graph: Pair Mid
graph twoway (rarea ivqr_pmid_c0 ivqr_pmid_c1 qt, col(gs10) fi(inten20)) ///
			 (line ivqr_pmid_est qt, lc(gs0)) ///
			 (line qr_pmid_est qt, lc(gs5) lp(dash)) ///
			 (line y qt, lc(gs3) lp(solid)), saving(fig1, replace) ///
xtitle("Quantile") ytitle("Coefficient") legend(off) yline(0, lc(gs3)) ///
title("Pair's Midterm")

* Graph: Own Mid
graph twoway (rarea ivqr_mid_c0 ivqr_mid_c1 qt, col(gs10) fi(inten20)) ///
			 (line ivqr_mid_est qt, lc(gs0)) ///
			 (line qr_mid_est qt, lc(gs5) lp(dash)) ///
			 (line y qt, lc(gs3) lp(solid)), saving(fig2, replace) ///
xtitle("Quantile") ytitle("Coefficient") yline(0, lc(gs3)) ///
title("Own Midterm") ///
legend(region(lcolor(none)) ring(0) pos(1) cols(1) lab(1 "IVQR 90% Conf. Interval") lab(2 "IVQR Coff.") lab(3 "QR Coeff.") order(1 2 3))

graph combine fig1.gph fig2.gph, cols(2) saving(fig_ivqr_0, replace)   /* change the name of file according to subgroup */
