function [bhat1,g,alpha]=inv_qr_oid(y,D,X,Z,tau,A)
% [bhat1,g,alpha,vc1,betas]=inv_qr(y,D,X,Z,tau,A)
% Note, all inputs are required.  If there are no covariates or instruments, pass [].
%  y - dependent variable
%  D - endogenous RHS variable (NOTE: This implementation requires dim(D) = 1)
%  X - covariates
%  Z - excluded instruments (NOTE: This implementation takes the linear projection of D on {X,Z}
%      and uses that as the instrument.)
%  tau - quantile index (tau in (0,1))
%  A - grid search region.  If A is not specified the grid search region is constructed
%      with centered on the 2SQR region.  Note that A must be a column vector.
% Output.
%  bhat1 - coefficient estimates. 
%  g - norm(gamma(alpha))
%  alpha - alpha.
%      NOTE: After estimation, plot(alpha,g) graphs objective function.


if nargin == 5,
    A = [];
end

X = [X,ones(size(y))];

x = [D,X];
z = [Z,X];
n = size(y,1);

if isempty(A),   % Calculate grid search region using 2SQR if no region is supplied.
    dhat     = z*inv(z'*z)*(z'*D);
    PSI      = [dhat,X];
    two_sqr  = rq(PSI,y,tau);
    if isnan(two_sqr),
        two_sqr = rq(PSI,y,tau+.00000001);
    end
    e1       = y-x*two_sqr;
    mu1      = mean(e1);
    sigma1   = var(e1);
    vc1      = ([tau*(1-tau)]/(normpdf(norminv(tau,mu1,sqrt(sigma1)),mu1,sqrt(sigma1)))^2)*inv(PSI'*PSI);
    std_2sqr = sqrt((n/100)*diag(vc1));
    if size(D,2) == 1,
        alpha = [two_sqr(1,1)-2*std_2sqr(1,1):std_2sqr(1,1)/50:two_sqr(1,1)+2*std_2sqr(1,1)]';
    else
        alpha1 = [two_sqr(1,1)-2*std_2sqr(1,1):std_2sqr(1,1)/5:two_sqr(1,1)+2*std_2sqr(1,1)]';
        alpha2 = [two_sqr(2,1)-2*std_2sqr(2,1):std_2sqr(2,1)/5:two_sqr(2,1)+2*std_2sqr(2,1)]';
    end    
    clear PSI two_sqr e1 mu1 sigma1 vc1 std_2sqr 
else
    if size(D,2) == 1,
        alpha = A;
    else
        alpha1 = A(:,1);
        alpha2 = A(:,2);
    end
end

dhat = Z;
z = [dhat,X];


if size(D,2) == 1,
    for i = 1:size(alpha,1),
        disp(i);
        betas(:,i) = rq(z,y-alpha(i,1)*D,tau); %#ok<AGROW>
        [temp,vc]  = vcqr_noconstant(betas(:,i),y-alpha(i,1)*D,z,tau);
        g(i,1)     = betas(1:size(dhat,2),i)'*inv(vc(1:size(dhat,2),1:size(dhat,2)))*betas(1:size(dhat,2),i); %#ok<AGROW>
%        g(i,1)     = norm([betas(1:size(dhat,2),i)]);
    end
    I      = find(g == min(g));
    param1 = alpha(I);
    est1   = betas(size(dhat,2)+1:size(betas,1),I);
    bhat1  = [param1;est1];
else
    for i = 1:size(alpha1,1),
        for j = 1:size(alpha2,1),
            alpha        = [alpha1(i,1) ; alpha2(j,1)];
            betas(:,i,j) = rq(z,y - D*alpha,tau); %#ok<AGROW>
            [temp,vc]  = vcqr_noconstant(betas(:,i,j),y-D*alpha,z,tau);
            g(i,j)       = betas(1:size(dhat,2),i,j)'*inv(vc(1:size(dhat,2),1:size(dhat,2)))*betas(1:size(dhat,2),i,j); %#ok<AGROW>
        end
    end
    [I,J]  = find(g == min(min(g)));
    param1 = [alpha1(I,1) ; alpha2(J,1)];
    est1   = betas(size(dhat,2)+1:size(betas,1),I,J);
    bhat1  = [param1 ; est1];
end

% PSI=[dhat,X];
                                    % Calculate naive standard errors based on normality
                                    % and homoskedasticity.
% e1=y-x*hat1;
% mu1=mean(e1);
% sigma1=var(e1);
% vc1=([tau*(1-tau)]/(normpdf(norminv(tau,mu1,sqrt(sigma1)),mu1,sqrt(sigma1)))^2)*inv(PSI'*PSI);

% std_err1=sqrt(diag(vc1));
% bhat1=[hat1,std_err1];

